//
//  lariat-online/include/BERNFragment.h    (W.Badgett)
//
//  Description of the BERN raw data format
//
//  The event format is as such:
//  Per time frame:  
//        One BernFrameHeader struct, 
//        followed by nHits number of BernHit structs
//
//        Typically frames are ~ 100ms long
//

#include <iostream>
#include <vector>
#include <sys/timeb.h>

#ifndef _BERNFragment_h
#define _BERNFragment_h

class BERNFragment
{
 public:

 enum
 {
   MAX_CARDS = 32
 };

#pragma pack(push,1)

  typedef struct BernFrameHeader
  {
    uint32_t fragmentSize;
    uint32_t fragmentType;
    uint32_t nHits;
    struct timeb tStart;
    struct timeb tEnd;
  } BernFrameHeader_t;

  typedef struct BernHit
  {
    uint16_t mac5;
    uint16_t flags;
    uint16_t lostCPU;
    uint16_t lostFPGA;
    uint32_t T1;
    uint32_t T2;
    uint16_t adc[32];
  } BernHit_t;


#pragma pack(pop)

  // the unpacked object data

  BernFrameHeader_t header;
  std::vector<BernHit_t> hits;

  /**
     * stream the data from this object into a buffer.
     * Returns the number of characters written as a convenience.
     * This function sets fragmentSize and fragmentType.
     **/
  uint32_t streamToBuffer(std::ostream& buffer);
  
  /**
   * Fill in this object, assuming that the current position
   * in the buffer coincides with the start of this object.
   * Returns the number of characters read as a convenience.
   * Obviously streamToBuffer() and fillFromBuffer() must agree on
   * the structure of the object in the buffer!
   **/
  uint32_t fillFromBuffer(std::istream& buffer);

    /**
   * print this object in a user readable format
   **/
  void print(std::ostream& os = std::cout);
};

std::ostream& operator<<(std::ostream& os, const BERNFragment& e);
std::ostream& operator<<(std::ostream& os, const BERNFragment::BernHit& e);

#endif
