//
//  lariat-online-lariatfragments/LArIATFragments/BERNFragment.cc
//

#include "BERNFragment.h"
#include "LariatFragment.h"
#include <stdexcept>
#include <sstream>

uint32_t BERNFragment::streamToBuffer(std::ostream& buffer)
{
  uint32_t nBytes=0;
    
  header.fragmentType = LariatFragment::FRAGMENT_TYPE_BERN_FRAGMENT;

  using std::streamsize;
  const streamsize headerSize = sizeof(BernFrameHeader_t);
  const streamsize hitSize    = sizeof(BernHit_t);
  
  header.fragmentSize = headerSize + hitSize*header.nHits;

  // write the header

  buffer.write(reinterpret_cast<const char*>(&header),headerSize);
  nBytes += headerSize;
  
  // write the channel hit data
  if(header.nHits!=hits.size()) 
  {
    throw std::length_error("BERNFragment::streamToBuffer : header.nHits!=hits.size()");
  }

  for(size_t i=0; i<header.nHits; ++i)
  {
    BernHit_t data=hits[i];
    buffer.write(reinterpret_cast<const char*>(&data),hitSize);
    nBytes += hitSize;
  }

  return nBytes;
}


uint32_t BERNFragment::fillFromBuffer(std::istream& buffer)
{
  uint32_t nBytes =0;
  
  using std::streamsize;
  const streamsize headerSize=sizeof(BernFrameHeader_t);
  const streamsize hitSize = sizeof(BernHit_t);
 
  
  //read the header

  buffer.read(reinterpret_cast<char*>(&header),headerSize);
  nBytes +=headerSize;

  //read the channel hit data

  for(size_t i=0; i<header.nHits; ++i)
  {
    BernHit_t data;
    buffer.read(reinterpret_cast<char*>(&data),hitSize);
    nBytes += hitSize;
    hits.push_back(data);
  }

  if(header.nHits!=hits.size()) 
  {
    throw std::length_error("BERNFragment::fillFromBuffer : header.nhits!=hits.size()");
  }  

  return nBytes ;
}

void BERNFragment::print(std::ostream& os)
{
  os << *this;
}

std::ostream& operator<<(std::ostream& os, const BERNFragment& e)
{
  os << "BernHitFrame" << std::endl
     << "  FragmentSize         " << e.header.fragmentSize << std::endl
     << "  FragmentType         " << e.header.fragmentType << std::endl
     << "  nHits                " << e.header.nHits        << std::endl;

  char line[256];
  sprintf(line, "  TimeStart            %d.%3.3d", 
	  (int)e.header.tStart.time,
	  (int)e.header.tStart.millitm);
  os << line << std::endl;
  sprintf(line, "  TimeEnd              %d.%3.3d", 
	  (int)e.header.tEnd.time,
	  (int)e.header.tEnd.millitm);
  os << line << std::endl;

  for(size_t iHit=0; iHit<e.header.nHits; ++iHit)
  { 
    os << e.hits[iHit] << std::endl;
  }
  return(os);
}

std::ostream& operator<<(std::ostream& os, const BERNFragment::BernHit& e)
{
  char buffer[256];
  snprintf(buffer,256,"MAC5 %d Flags 0x%X T1 %d T2 %d LostCPU %d LostFPGA %d",
	   e.mac5, e.flags, e.T1, e.T2, e.lostCPU, e.lostFPGA);
  os << buffer;
  return(os);
}

