//
//   lariat-online/daq/src/TDCFragment   (W.Badgett)
//

#include "TDCFragment.h"
#include <stdexcept>
#include <iostream>
#include <fstream>

uint32_t TDCFragment::streamToBuffer(std::ostream& buffer)
{
  return(0);
}

uint32_t TDCFragment::fillFromBuffer(std::istream& buffer)
{
  uint32_t bytes = 0;

  buffer.read(reinterpret_cast<char*>(&controllerHeader),
	      sizeof(struct TdcControllerHeader));
  if ( buffer.eof() ) { return(bytes);}

  bytes += sizeof(struct TdcControllerHeader);
  //  printTdcControllerHeader(controllerHeader);

  for ( size_t i=0; i<controllerHeader.nTDCs; ++i)
  {
    TdcSpillHeader tdcSpillHeader;
    buffer.read(reinterpret_cast<char*>(&tdcSpillHeader), sizeof(struct TdcSpillHeader));
    if ( buffer.eof() ) { return(bytes);}

    bytes += sizeof(struct TdcSpillHeader);
    if ( tdcSpillHeader.spillTriggerCounter != 
	 controllerHeader.triggerCounts )
    {
      char text[1024];
      sprintf(text,"Fatal error: TDC has %d triggers, controller has %d",
	      tdcSpillHeader.spillTriggerCounter,
	      controllerHeader.triggerCounts );
      // throw(std::length_error(text));
    }
    //    printTdcSpillHeader(tdcSpillHeader);
    tdcSpillHeaders.push_back(tdcSpillHeader);
  }

  for ( size_t i=0; i<controllerHeader.triggerCounts; ++i)
  {
    std::vector<TdcEventData> tdcEventVector;
    for ( size_t j=0; j<controllerHeader.nTDCs; ++j)
    {
      TdcEventData tdcEventData;
      buffer.read(reinterpret_cast<char*>(&tdcEventData.tdcEventHeader), 
		  sizeof(struct TdcEventHeader));
      if ( buffer.eof() ) { return(bytes);}
      bytes += sizeof(struct TdcEventHeader);
      //printTdcEventHeader(tdcEventData.tdcEventHeader);
      if ( tdcEventData.tdcEventHeader.tdcNumber != (char)(j+1))
      {
	std::cout << "**** Fatal mismatch " << (int)tdcEventData.tdcEventHeader.tdcNumber
		  << " != " << (j+1) << std::endl;
      }

      for ( size_t k=0; k<tdcEventData.tdcEventHeader.nHits; ++k)
      {
	struct TdcHit tdcHit;
	buffer.read(reinterpret_cast<char*>(&tdcHit), sizeof(struct TdcHit));
	if ( buffer.eof() ) { return(bytes);}
	bytes += sizeof(struct TdcHit);

	tdcEventData.tdcHits.push_back(tdcHit);
      }
      tdcEventVector.push_back(tdcEventData);
    }
    tdcEvents.push_back(tdcEventVector);
  }
  return(bytes);
}

void TDCFragment::printTdcControllerHeader(TdcControllerHeader header,
					   std::ostream & os)
{
  os << header;
}

void TDCFragment::print(std::ostream & os)
{
  os << *this;
}

 void TDCFragment::printTdcSpillHeader(TdcSpillHeader header,
				       std::ostream & os)
{
  os << header;
}

void TDCFragment::printTdcEventHeader(TdcEventHeader header,
				      std::ostream & os)
{
  os << header;
}

void TDCFragment::printTdcHit(TdcHit hit, uint32_t index,
			      std::ostream & os)
{
  char text[256];
  sprintf(text,"  Hit %3d  Channel %5d   TimeBin %5d\n", index,
	 hit.channel, hit.timeBin);
  os << text;
}

std::ostream& operator<<(std::ostream& os, const TDCFragment::TdcHit& e)
{
  char text[256];
  sprintf(text,"Channel %5d   TimeBin %5d\n", 
	 e.channel, e.timeBin);
  os << text;
  return(os);
}

std::ostream& operator<<(std::ostream& os, const TDCFragment::TdcEventHeader& e)
{
  os << "TdcEventHeader" << std::endl
     << "  FragmentSize         " << e.fragmentSize<< std::endl
     << "  FragmentType         " << e.fragmentType<< std::endl
     << "  EventWordCount       " << (int)e.eventWordCount<< std::endl
     << "  ControllerNumber     " << (int)e.controllerNumber<< std::endl
     << "  TdcNumber            " << (int)e.tdcNumber<< std::endl
     << "  EventStatus          " << (int)e.eventStatus<< std::endl
     << "  TriggerCounter       " << e.triggerCounter<< std::endl
     << "  TriggerType          " << (int)e.triggerType<< std::endl
     << "  ControllerTimeStamp  " << e.controllerTimeStamp<< std::endl
     << "  TdcTimeStamp         " << e.tdcTimeStamp<< std::endl
     << "  nHits                " << e.nHits;
  return(os);
}

std::ostream& operator<<(std::ostream& os, const TDCFragment::TdcControllerHeader& e)
{
  os << "TdcControllerHeader" << std::endl
   << "  FragmentSize         " << e.fragmentSize<< std::endl
   << "  FragmentType         " << e.fragmentType<< std::endl
   << "  SpillWordCount       " << e.spillWordCount<< std::endl
   << "  SpillWordCountBuffer " << e.spillWordCountBuffer<< std::endl
   << "  SpillCounter         " << e.spillCounter<< std::endl
   << "  ControllerNumber     " << (int)e.controllerNumber<< std::endl
   << "  Year                 " << (int)e.year<< std::endl
   << "  Month                " << (int)e.month<< std::endl
   << "  Day                  " << (int)e.day<< std::endl
   << "  Hours                " << (int)e.hours<< std::endl
   << "  Minutes              " << (int)e.minutes<< std::endl
   << "  Seconds              " << (int)e.seconds<< std::endl
   << "  TriggerCounts        " << e.triggerCounts<< std::endl
   << "  TdcStatus            0x" << std::hex << e.tdcStatus
   << std::endl
   << "  LinkStatus           0x" << std::hex << e.linkStatus 
   << std::endl
   << "  TdcActiveMask        0x" << std::hex << e.tdcActiveMask 
   << std::endl
   << "  nTDCs                " << std::dec << e.nTDCs;
  return(os);
}

std::ostream& operator<<(std::ostream& os, const TDCFragment::TdcSpillHeader& e)
{
  os << "TdcSpillHeader" << std::endl
     << "  FragmentSize         " << e.fragmentSize<< std::endl
     << "  FragmentType         " << e.fragmentType<< std::endl 
     << "  SpillWordCount       " << e.spillWordCount<< std::endl 
     << "  SpillTriggerCounter  " << e.spillTriggerCounter<< std::endl
     << "  SpillStatus          0x" << std::hex << e.spillStatus << std::endl
     << "  ControllerNumber     " << std::dec << (int)e.controllerNumber<< std::endl
     << "  TdcNumber            " << (int)e.tdcNumber;
  return(os);
}

std::ostream& operator<<(std::ostream& os, const TDCFragment& e)
{
  os <<e.controllerHeader
     << "  tdcSpillHeader.size()" << e.tdcSpillHeaders.size() << std::endl
     << "  tdcEvents.size()     " << e.tdcEvents.size() << std::endl;
  for ( auto &i: e.tdcSpillHeaders )
  {
    os << i;
  }

  // Loop over events/triggers first, one entry per event
  for (auto &i: e.tdcEvents )
  {
    // Here "i" is std::vector<TdcEventData> which should have one entry per TDC
    // Next loop over TDCs within the event
    for ( auto &j: i )
    {
      // Now "j" is TdcEventData class

      os << j.tdcEventHeader;
      int h=0;
      for ( auto &k: j.tdcHits )
      {
	char text[256];
	sprintf(text,"  Hit %3d  ", h);
	os << text << k;
	h++;
      }
    }
  }
  return(os);
}

